@@ -32,9 +32,9 @@
 
 from __future__ import annotations
 
-import importlib
 import os
 import re
+import warnings
 import weakref
 
 from abc import ABC, abstractmethod
@@ -80,6 +80,7 @@ from .constants import (
 )
 from .conversion import MySQLConverter, MySQLConverterBase
 from .errors import (
+    DatabaseError,
     Error,
     InterfaceError,
     NotSupportedError,
@@ -110,6 +111,7 @@ from .types import (
     SupportedMysqlBinaryProtocolTypes,
     WarningType,
 )
+from .utils import import_object
 
 NAMED_TUPLE_CACHE: weakref.WeakValueDictionary[Any, Any] = weakref.WeakValueDictionary()
 
@@ -186,7 +188,8 @@ class MySQLConnectionAbstract(ABC):
         self._force_ipv6: bool = False
         self._oci_config_file: Optional[str] = None
         self._oci_config_profile: Optional[str] = None
-        self._fido_callback: Optional[Union[str, Callable]] = None
+        self._fido_callback: Optional[Union[str, Callable[[str], None]]] = None
+        self._webauthn_callback: Optional[Union[str, Callable[[str], None]]] = None
         self._krb_service_principal: Optional[str] = None
 
         self._use_unicode: bool = True
@@ -219,6 +222,7 @@ class MySQLConnectionAbstract(ABC):
 
         self._consume_results: bool = False
         self._init_command: Optional[str] = None
+        self._character_set: CharacterSet = CharacterSet()
 
     def __enter__(self) -> MySQLConnectionAbstract:
         return self
@@ -577,19 +581,12 @@ class MySQLConnectionAbstract(ABC):
         except KeyError:
             self._auth_plugin = ""
 
-        # Configure character set and collation
-        if "charset" in config or "collation" in config:
-            try:
-                charset = config["charset"]
-                del config["charset"]
-            except KeyError:
-                charset = None
-            try:
-                collation = config["collation"]
-                del config["collation"]
-            except KeyError:
-                collation = None
-            self._charset_id = CharacterSet.get_charset_info(charset, collation)[0]
+        # Disallow the usage of some default authentication plugins
+        if self._auth_plugin == "authentication_webauthn_client":
+            raise InterfaceError(
+                f"'{self._auth_plugin}' cannot be used as the default authentication "
+                "plugin"
+            )
 
         # Set converter class
         try:
@@ -794,33 +791,55 @@ class MySQLConnectionAbstract(ABC):
                 )
 
         if self._fido_callback:
-            # Import the callable if it's a str
-            if isinstance(self._fido_callback, str):
-                try:
-                    module, callback = self._fido_callback.rsplit(".", 1)
-                except ValueError:
-                    raise ProgrammingError(
-                        f"No callable named '{self._fido_callback}'"
-                    ) from None
-                try:
-                    module = importlib.import_module(module)
-                    self._fido_callback = getattr(module, callback)
-                except (AttributeError, ModuleNotFoundError) as err:
-                    raise ProgrammingError(f"{err}") from err
-            # Check if it's a callable
-            if not callable(self._fido_callback):
-                raise ProgrammingError("Expected a callable for 'fido_callback'")
-            # Check the callable signature if has only 1 positional argument
-            params = len(signature(self._fido_callback).parameters)
-            if params != 1:
-                raise ProgrammingError(
-                    "'fido_callback' requires 1 positional argument, but the "
-                    f"callback provided has {params}"
-                )
+            warn_msg = (
+                "The `fido_callback` connection argument is deprecated and it will be "
+                "removed in a future release of MySQL Connector/Python. "
+                "Use `webauth_callback` instead"
+            )
+            warnings.warn(warn_msg, DeprecationWarning)
+            self._validate_callable("fido_callback", self._fido_callback, 1)
+
+        if self._webauthn_callback:
+            self._validate_callable("webauth_callback", self._webauthn_callback, 1)
 
     def _add_default_conn_attrs(self) -> Any:
         """Add the default connection attributes."""
 
+    @staticmethod
+    def _validate_callable(
+        option_name: str, callback: Union[str, Callable], num_args: int = 0
+    ) -> None:
+        """Validate if it's a Python callable.
+
+         Args:
+             option_name (str): Connection option name.
+             callback (str or callable): The fully qualified path to the callable or
+                                         a callable.
+             num_args (int): Number of positional arguments allowed.
+
+        Raises:
+             ProgrammingError: If `callback` is not valid or wrong number of positional
+                               arguments.
+
+        .. versionadded:: 8.2.0
+        """
+        if isinstance(callback, str):
+            try:
+                callback = import_object(callback)
+            except ValueError as err:
+                raise ProgrammingError(f"{err}") from err
+
+        if not callable(callback):
+            raise ProgrammingError(f"Expected a callable for '{option_name}'")
+
+        # Check if the callable signature has <num_args> positional arguments
+        num_params = len(signature(callback).parameters)
+        if num_params != num_args:
+            raise ProgrammingError(
+                f"'{option_name}' requires {num_args} positional argument, but the "
+                f"callback provided has {num_params}"
+            )
+
     @staticmethod
     def _check_server_version(server_version: StrOrBytes) -> Tuple[int, ...]:
         """Check the MySQL version
@@ -1077,7 +1096,7 @@ class MySQLConnectionAbstract(ABC):
 
         Returns a string.
         """
-        return CharacterSet.get_info(self._charset_id)[0]
+        return self._character_set.get_info(self._charset_id)[0]
 
     @property
     def python_charset(self) -> str:
@@ -1090,7 +1109,7 @@ class MySQLConnectionAbstract(ABC):
 
         Returns a string.
         """
-        encoding = CharacterSet.get_info(self._charset_id)[0]
+        encoding = self._character_set.get_info(self._charset_id)[0]
         if encoding in ("utf8mb4", "utf8mb3", "binary"):
             return "utf8"
         return encoding
@@ -1126,13 +1145,13 @@ class MySQLConnectionAbstract(ABC):
                     self._charset_id,
                     charset_name,
                     collation_name,
-                ) = CharacterSet.get_charset_info(charset)
+                ) = self._character_set.get_charset_info(charset)
             elif isinstance(charset, str):
                 (
                     self._charset_id,
                     charset_name,
                     collation_name,
-                ) = CharacterSet.get_charset_info(charset, collation)
+                ) = self._character_set.get_charset_info(charset, collation)
             else:
                 raise ValueError(err_msg.format("charset"))
         elif collation:
@@ -1140,14 +1159,14 @@ class MySQLConnectionAbstract(ABC):
                 self._charset_id,
                 charset_name,
                 collation_name,
-            ) = CharacterSet.get_charset_info(collation=collation)
+            ) = self._character_set.get_charset_info(collation=collation)
         else:
             charset = DEFAULT_CONFIGURATION["charset"]
             (
                 self._charset_id,
                 charset_name,
                 collation_name,
-            ) = CharacterSet.get_charset_info(charset, collation=None)
+            ) = self._character_set.get_charset_info(charset, collation=None)
 
         self._execute_query(f"SET NAMES '{charset_name}' COLLATE '{collation_name}'")
 
@@ -1159,7 +1178,7 @@ class MySQLConnectionAbstract(ABC):
             pass
 
         if self.converter:
-            self.converter.set_charset(charset_name)
+            self.converter.set_charset(charset_name, character_set=self._character_set)
 
     @property
     def collation(self) -> str:
@@ -1171,7 +1190,7 @@ class MySQLConnectionAbstract(ABC):
 
         Returns a string.
         """
-        return CharacterSet.get_charset_info(self._charset_id)[2]
+        return self._character_set.get_charset_info(self._charset_id)[2]
 
     @abstractmethod
     def _do_handshake(self) -> Any:
@@ -1210,15 +1229,41 @@ class MySQLConnectionAbstract(ABC):
         arguments are given, it will use the already configured or default
         values.
         """
+        # open connection using the default charset id
         if kwargs:
             self.config(**kwargs)
 
         self.disconnect()
         self._open_connection()
-        # Server does not allow to run any other statement different from ALTER
-        # when user's password has been expired.
+
+        charset, collation = (
+            kwargs.pop("charset", None),
+            kwargs.pop("collation", None),
+        )
+        if charset or collation:
+            self._charset_id = self._character_set.get_charset_info(charset, collation)[
+                0
+            ]
+
         if not self._client_flags & ClientFlag.CAN_HANDLE_EXPIRED_PASSWORDS:
             self._post_connection()
+        else:
+            # the server does not allow to run any other statement different from
+            # ALTER when user's password has been expired - the server either
+            # disconnects the client or restricts the client to "sandbox mode" [1].
+            # [1]: https://dev.mysql.com/doc/refman/5.7/en/expired-password-handling.html
+            try:
+                self.set_charset_collation(charset=self._charset_id)
+            except DatabaseError:
+                # get out of sandbox mode - with no FOR user clause, the statement sets
+                # the password for the current user.
+                self.cmd_query(f"SET PASSWORD = '{self._password1 or self._password}'")
+
+                # Set charset and collation.
+                self.set_charset_collation(charset=self._charset_id)
+
+                # go back to sandbox mode.
+                self.cmd_query("ALTER USER CURRENT_USER() PASSWORD EXPIRE")
 
     def reconnect(self, attempts: int = 1, delay: int = 0) -> None:
         """Attempt to reconnect to the MySQL server
@@ -1413,7 +1458,7 @@ class MySQLConnectionAbstract(ABC):
         methods and members of conversion.MySQLConverter.
         """
         if convclass and issubclass(convclass, MySQLConverterBase):
-            charset_name = CharacterSet.get_info(self._charset_id)[0]
+            charset_name = self._character_set.get_info(self._charset_id)[0]
             self._converter_class = convclass
             self.converter = convclass(charset_name, self._use_unicode)
             self.converter.str_fallback = self._converter_str_fallback
