@@ -33,12 +33,13 @@
 import os
 import platform
 import socket
+import sys
 
 from typing import Any, Dict, List, Optional, Sequence, Tuple, Type, Union
 
 from . import version
 from .abstracts import MySQLConnectionAbstract
-from .constants import CharacterSet, ClientFlag, FieldFlag, ServerFlag, ShutdownType
+from .constants import ClientFlag, FieldFlag, ServerFlag, ShutdownType
 from .conversion import MySQLConverter
 from .errors import (
     InterfaceError,
@@ -57,6 +58,7 @@ from .types import (
     StatsPacketType,
     StrOrBytes,
 )
+from .utils import import_object
 
 HAVE_CMYSQL = False
 
@@ -86,6 +88,12 @@ except ImportError as exc:
         f"MySQL Connector/Python C Extension not available ({exc})"
     ) from exc
 
+from .opentelemetry.constants import OTEL_ENABLED
+from .opentelemetry.context_propagation import with_context_propagation
+
+if OTEL_ENABLED:
+    from .opentelemetry.instrumentation import end_span, record_exception_event
+
 
 class CMySQLConnection(MySQLConnectionAbstract):
     """Class initiating a MySQL Connection using Connector/C."""
@@ -117,7 +125,11 @@ class CMySQLConnection(MySQLConnectionAbstract):
         super().__init__()
 
         if kwargs:
-            self.connect(**kwargs)
+            try:
+                self.connect(**kwargs)
+            except Exception:
+                self.close()
+                raise
 
     def _add_default_conn_attrs(self) -> None:
         """Add default connection attributes"""
@@ -152,7 +164,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
         self._server_version = self._check_server_version(
             self._handshake["server_version_original"]
         )
-        CharacterSet.set_mysql_version(self._server_version)
+        self._character_set.set_mysql_version(self._server_version)
 
     @property
     def _server_status(self) -> int:
@@ -218,7 +230,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
         return self._server_status & ServerFlag.STATUS_IN_TRANS
 
     def _open_connection(self) -> None:
-        charset_name = CharacterSet.get_info(self._charset_id)[0]
+        charset_name = self._character_set.get_info(self._charset_id)[0]
         # pylint: disable=c-extension-no-member
         self._cmysql = _mysql_connector.MySQL(
             buffered=self._buffered,
@@ -232,6 +244,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
         # pylint: enable=c-extension-no-member
         if not self.isset_client_flag(ClientFlag.CONNECT_ARGS):
             self._conn_attrs = {}
+        fido_callback = self._webauthn_callback or self._fido_callback
         cnx_kwargs = {
             "host": self._host,
             "user": self._user,
@@ -243,14 +256,18 @@ class CMySQLConnection(MySQLConnectionAbstract):
             "port": self._port,
             "client_flags": self._client_flags,
             "unix_socket": self._unix_socket,
-            "compress": self.isset_client_flag(ClientFlag.COMPRESS),
+            "compress": self._compress,
             "ssl_disabled": True,
             "conn_attrs": self._conn_attrs,
             "local_infile": self._allow_local_infile,
             "load_data_local_dir": self._allow_local_infile_in_path,
             "oci_config_file": self._oci_config_file,
             "oci_config_profile": self._oci_config_profile,
-            "fido_callback": self._fido_callback,
+            "fido_callback": (
+                import_object(fido_callback)
+                if isinstance(fido_callback, str)
+                else fido_callback
+            ),
         }
 
         tls_versions = self._ssl.get("tls_versions")
@@ -301,14 +318,24 @@ class CMySQLConnection(MySQLConnectionAbstract):
 
     def close(self) -> None:
         """Disconnect from the MySQL server"""
-        if self._cmysql:
-            try:
-                self.free_result()
-                self._cmysql.close()
-            except MySQLInterfaceError as err:
-                raise get_mysql_exception(
-                    msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
-                ) from err
+        if self._span and self._span.is_recording():
+            record_exception_event(self._span, sys.exc_info()[1])
+
+        if not self._cmysql:
+            return
+
+        try:
+            self.free_result()
+            self._cmysql.close()
+        except MySQLInterfaceError as err:
+            if OTEL_ENABLED:
+                record_exception_event(self._span, err)
+            raise get_mysql_exception(
+                msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+            ) from err
+        finally:
+            if OTEL_ENABLED:
+                end_span(self._span)
 
     disconnect = close
 
@@ -594,6 +621,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
             raise InternalError("Unread result found")
         statement_id.stmt_reset()
 
+    @with_context_propagation
     def cmd_query(
         self,
         query: StrOrBytes,
@@ -613,7 +641,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
                 raw=raw,
                 buffered=buffered,
                 raw_as_string=raw_as_string,
-                query_attrs=self._query_attrs,
+                query_attrs=self.query_attrs,
             )
         except MySQLInterfaceError as err:
             raise get_mysql_exception(
@@ -836,6 +864,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
             ) from err
 
         self._charset_id = charset
+        self._user = username  # updating user accordingly
         self._post_connection()
 
     def cmd_reset_connection(self) -> bool:
