@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2009, 2023, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -65,6 +65,7 @@ class MySQLConverterBase:
         use_unicode: bool = True,
         str_fallback: bool = False,
     ) -> None:
+        self._character_set: CharacterSet = CharacterSet()
         self.python_types: Optional[Tuple[Any, ...]] = None
         self.mysql_types: Optional[Tuple[Any, ...]] = None
         self.charset: Optional[str] = None
@@ -77,7 +78,9 @@ class MySQLConverterBase:
             Callable[[bytes, DescriptionType], ToPythonOutputTypes],
         ] = {}
 
-    def set_charset(self, charset: Optional[str]) -> None:
+    def set_charset(
+        self, charset: Optional[str], character_set: Optional[CharacterSet] = None
+    ) -> None:
         """Set character set"""
         if charset in ("utf8mb4", "utf8mb3"):
             charset = "utf8"
@@ -86,7 +89,11 @@ class MySQLConverterBase:
         else:
             # default to utf8
             self.charset = "utf8"
-        self.charset_id = CharacterSet.get_charset_info(self.charset)[0]
+
+        if character_set:
+            self._character_set = character_set
+
+        self.charset_id = self._character_set.get_charset_info(self.charset)[0]
 
     def set_unicode(self, value: bool = True) -> None:
         """Set whether to use Unicode"""
@@ -132,7 +139,10 @@ class MySQLConverterBase:
             return value
 
     @staticmethod
-    def escape(value: Any) -> Any:
+    def escape(
+        value: Any,
+        sql_mode: Optional[str] = None,  # pylint: disable=unused-argument
+    ) -> Any:
         """Escape buffer for sending to MySQL"""
         return value
 
@@ -162,14 +172,14 @@ class MySQLConverter(MySQLConverterBase):
         use_unicode: bool = True,
         str_fallback: bool = False,
     ) -> None:
-        MySQLConverterBase.__init__(self, charset, use_unicode, str_fallback)
+        super().__init__(charset, use_unicode, str_fallback)
         self._cache_field_types: Dict[
             int,
             Callable[[bytes, DescriptionType], ToPythonOutputTypes],
         ] = {}
 
     @staticmethod
-    def escape(value: Any) -> Any:
+    def escape(value: Any, sql_mode: Optional[str] = None) -> Any:
         """
         Escapes special characters as they are expected to by when MySQL
         receives them.
@@ -178,6 +188,8 @@ class MySQLConverter(MySQLConverterBase):
         Returns the value if not a string, or the escaped string.
         """
         if isinstance(value, (bytes, bytearray)):
+            if sql_mode == "NO_BACKSLASH_ESCAPES":
+                return value.replace(b"'", b"''")
             value = value.replace(b"\\", b"\\\\")
             value = value.replace(b"\n", b"\\n")
             value = value.replace(b"\r", b"\\r")
@@ -185,6 +197,8 @@ class MySQLConverter(MySQLConverterBase):
             value = value.replace(b"\042", b"\134\042")  # double quotes
             value = value.replace(b"\032", b"\134\032")  # for Win32
         elif isinstance(value, str) and not isinstance(value, HexLiteral):
+            if sql_mode == "NO_BACKSLASH_ESCAPES":
+                return value.replace("'", "''")
             value = value.replace("\\", "\\\\")
             value = value.replace("\n", "\\n")
             value = value.replace("\r", "\\r")
@@ -270,8 +284,8 @@ class MySQLConverter(MySQLConverterBase):
     def _long_to_mysql(value: int) -> int:
         """Convert value to int
 
-        Note: there is not type "long" in Python 3 since integers (int) are of unlimited size.
-        Since Python 2 is no longer supported, this method should be deprecated.
+        Note: There is no type "long" in Python 3 since integers are of unlimited size.
+              Since Python 2 is no longer supported, this method should be deprecated.
         """
         return int(value)
 
@@ -292,9 +306,9 @@ class MySQLConverter(MySQLConverterBase):
         charset_id = self.charset_id
         if charset == "binary":
             charset = "utf8"
-            charset_id = CharacterSet.get_charset_info(charset)[0]
+            charset_id = self._character_set.get_charset_info(charset)[0]
         encoded = value.encode(charset)
-        if charset_id in CharacterSet.slash_charsets:
+        if charset_id in self._character_set.slash_charsets:
             if b"\x5c" in encoded:
                 return HexLiteral(value, charset)
         return encoded
