@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2009, 2023, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -86,10 +86,10 @@ RE_SQL_ON_DUPLICATE = re.compile(
 RE_SQL_INSERT_STMT = re.compile(
     rf"({SQL_COMMENT}|\s)*INSERT({SQL_COMMENT}|\s)"
     r"*(?:IGNORE\s+)?INTO\s+[`'\"]?.+[`'\"]?(?:\.[`'\"]?.+[`'\"]?)"
-    r"{0,2}\s+VALUES\s*\(.+(?:\s*,.+)*\)",
+    r"{0,2}\s+VALUES\s*(\(.+\)).*",
     re.I | re.M | re.S,
 )
-RE_SQL_INSERT_VALUES = re.compile(r".*VALUES\s*(\(.*\)).*", re.I | re.M | re.S)
+RE_SQL_INSERT_VALUES = re.compile(r".*VALUES\s*(\(.+\)).*", re.I | re.M | re.S)
 RE_PY_PARAM = re.compile(b"(%s)")
 RE_PY_MAPPING_PARAM = re.compile(
     rb"""
@@ -99,7 +99,9 @@ RE_PY_MAPPING_PARAM = re.compile(
     """,
     re.X,
 )
-RE_SQL_SPLIT_STMTS = re.compile(b""";(?=(?:[^"'`]*["'`].*["'`])*[^"'`]*$)""")
+RE_SQL_SPLIT_STMTS = re.compile(
+    b""";(?=(?:[^"'`]*(?:"[^"]*"|'[^']*'|`[^`]*`))*[^"'`]*$)"""
+)
 RE_SQL_FIND_PARAM = re.compile(b"""%s(?=(?:[^"'`]*["'`][^"'`]*["'`])*[^"'`]*$)""")
 RE_SQL_PYTHON_REPLACE_PARAM = re.compile(r"%\(.*?\)s")
 RE_SQL_PYTHON_CAPTURE_PARAM_NAME = re.compile(r"%\((.*?)\)s")
@@ -434,15 +436,16 @@ class MySQLCursor(CursorBase):
         self, params: ParamsDictType
     ) -> Dict[bytes, Union[bytes, Decimal]]:
         """Process query parameters given as dictionary"""
+        res: Dict[bytes, Any] = {}
         try:
+            sql_mode = self._connection.sql_mode
             to_mysql = self._connection.converter.to_mysql
             escape = self._connection.converter.escape
             quote = self._connection.converter.quote
-            res: Dict[bytes, Any] = {}
             for key, value in params.items():
                 conv = value
                 conv = to_mysql(conv)
-                conv = escape(conv)
+                conv = escape(conv, sql_mode)
                 if not isinstance(value, Decimal):
                     conv = quote(conv)
                 res[key.encode()] = conv
@@ -450,22 +453,20 @@ class MySQLCursor(CursorBase):
             raise ProgrammingError(
                 f"Failed processing pyformat-parameters; {err}"
             ) from err
-        else:
-            return res
+        return res
 
     def _process_params(
         self, params: ParamsSequenceType
     ) -> Tuple[Union[bytes, Decimal], ...]:
         """Process query parameters."""
+        res = params[:]
         try:
-            res = params[:]
-
+            sql_mode = self._connection.sql_mode
             to_mysql = self._connection.converter.to_mysql
             escape = self._connection.converter.escape
             quote = self._connection.converter.quote
-
             res = [to_mysql(value) for value in res]
-            res = [escape(value) for value in res]
+            res = [escape(value, sql_mode) for value in res]
             res = [
                 quote(value) if not isinstance(params[i], Decimal) else value
                 for i, value in enumerate(res)
@@ -474,8 +475,7 @@ class MySQLCursor(CursorBase):
             raise ProgrammingError(
                 f"Failed processing format-parameters; {err}"
             ) from err
-        else:
-            return tuple(res)
+        return tuple(res)
 
     def _handle_noresultset(self, res: ResultType) -> None:
         """Handles result of execute() when there is no result set"""
@@ -1094,10 +1094,8 @@ class MySQLCursorBuffered(MySQLCursor):
             row = self._rows[self._next_row]
         except (IndexError, TypeError):
             return None
-        else:
-            self._next_row += 1
-            return row
-        return None
+        self._next_row += 1
+        return row
 
     def fetchone(self) -> Optional[RowType]:
         """Return next row of a query result set.
@@ -1549,13 +1547,14 @@ class MySQLCursorNamedTuple(MySQLCursor):
         Returns:
             tuple or None: A row from query result set.
         """
-        self._check_executed()
-        row = self._fetch_row()
-        if row:
-            if hasattr(self._connection, "converter"):
-                return self._row_to_python(row, self.description)
-            return row
-        return None
+        row = super().fetchone()
+        if not row:
+            return None
+        return (
+            self._row_to_python(row, self.description)
+            if hasattr(self._connection, "converter")
+            else row
+        )
 
     def fetchall(self) -> List[Optional[RowType]]:
         """Return all rows of a query result set.
@@ -1563,21 +1562,11 @@ class MySQLCursorNamedTuple(MySQLCursor):
         Returns:
             list: A list of tuples with all rows of a query result set.
         """
-        self._check_executed()
-        if not self._have_unread_result():
-            return []
-
-        (rows, eof) = self._connection.get_rows()
-        if self._nextrow[0]:
-            rows.insert(0, self._nextrow[0])
-        res = [self._row_to_python(row, self.description) for row in rows]
-
-        self._handle_eof(eof)
-        rowcount = len(rows)
-        if rowcount >= 0 and self._rowcount == -1:
-            self._rowcount = 0
-        self._rowcount += rowcount
-        return res
+        return [
+            self._row_to_python(row, self.description)
+            for row in super().fetchall()
+            if row
+        ]
 
 
 class MySQLCursorBufferedDict(MySQLCursorDict, MySQLCursorBuffered):
@@ -1680,3 +1669,88 @@ class MySQLCursorPreparedDict(MySQLCursorDict, MySQLCursorPrepared):  # type: ig
             for row in super().fetchmany(size=size)
             if row
         ]
+
+
+class MySQLCursorPreparedNamedTuple(MySQLCursorNamedTuple, MySQLCursorPrepared):
+    """
+    This class is a blend of features from MySQLCursorNamedTuple and MySQLCursorPrepared
+    """
+
+    def fetchmany(self, size: Optional[int] = None) -> List[RowType]:
+        """Return the next set of rows of a query result set.
+
+        When no more rows are available, it returns an empty list.
+        The number of rows returned can be specified using the size argument,
+        which defaults to one.
+
+        Returns:
+            list: The next set of rows of a query result set represented
+                  as a list of named tuples where column names are used as names.
+        """
+        return [
+            self._row_to_python(row, self.description)
+            for row in super().fetchmany(size=size)
+            if row
+        ]
+
+
+class MySQLCursorPreparedRaw(MySQLCursorPrepared):
+    """
+    This class is a blend of features from MySQLCursorRaw and MySQLCursorPrepared
+    """
+
+    _raw: bool = True
+
+    def fetchone(self) -> Optional[RowType]:
+        """Return next row of a query result set.
+
+        Returns:
+            tuple or None: A row from query result set.
+        """
+        self._check_executed()
+        if self._cursor_exists:
+            self._connection.cmd_stmt_fetch(self._prepared["statement_id"])
+        return self._fetch_row(raw=self._raw) or None
+
+    def fetchmany(self, size: Optional[int] = None) -> List[RowType]:
+        """Return the next set of rows of a query result set.
+
+        When no more rows are available, it returns an empty list.
+        The number of rows returned can be specified using the size argument,
+        which defaults to one.
+
+        Returns:
+            list: The next set of rows of a query result set.
+        """
+        self._check_executed()
+        res = []
+        cnt = size or self.arraysize
+        while cnt > 0 and self._have_unread_result():
+            cnt -= 1
+            row = self._fetch_row(raw=self._raw)
+            if row:
+                res.append(row)
+        return res
+
+    def fetchall(self) -> List[RowType]:
+        """Return all rows of a query result set.
+
+        Returns:
+            list: A list of tuples with all rows of a query result set.
+        """
+        self._check_executed()
+        rows = []
+        if self._nextrow[0]:
+            rows.append(self._nextrow[0])
+        while self._have_unread_result():
+            if self._cursor_exists:
+                self._connection.cmd_stmt_fetch(
+                    self._prepared["statement_id"], MAX_RESULTS
+                )
+            (tmp, eof) = self._connection.get_rows(
+                raw=self._raw, binary=self._binary, columns=self.description
+            )
+            rows.extend(tmp)
+            self._handle_eof(eof)
+        self._rowcount = len(rows)
+        return rows
