@@ -1,4 +1,4 @@
-# Copyright (c) 2016, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2016, 2023, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -28,11 +28,11 @@
 
 """Implementation of the X protocol for MySQL servers."""
 
-import logging
 import struct
 import zlib
 
 from io import BytesIO
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 try:
     import lz4.frame
@@ -63,17 +63,33 @@ from .expr import (
     build_unsigned_int_scalar,
 )
 from .helpers import encode_to_bytes, get_item_or_attr
-from .protobuf import (
-    CRUD_PREPARE_MAPPING,
-    PROTOBUF_REPEATED_TYPES,
-    SERVER_MESSAGES,
-    Message,
-    mysqlxpb_enum,
-)
+from .logger import logger
+from .protobuf import CRUD_PREPARE_MAPPING, SERVER_MESSAGES, Message, mysqlxpb_enum
 from .result import Column
+from .statement import (
+    AddStatement,
+    DeleteStatement,
+    FilterableStatement,
+    FindStatement,
+    InsertStatement,
+    ModifyStatement,
+    ReadStatement,
+    RemoveStatement,
+    SqlStatement,
+    UpdateStatement,
+)
+from .types import (
+    ColumnType,
+    MessageType,
+    ProtobufMessageCextType,
+    ProtobufMessageType,
+    ResultBaseType,
+    SocketType,
+    StatementType,
+    StrOrBytes,
+)
 
 _COMPRESSION_THRESHOLD = 1000
-_LOGGER = logging.getLogger("mysqlx")
 
 
 class Compressor:
@@ -87,19 +103,19 @@ class Compressor:
 
     """
 
-    def __init__(self, algorithm):
-        self._algorithm = algorithm
+    def __init__(self, algorithm: str) -> None:
+        self._algorithm: str = algorithm
+        self._compressobj: Any = None
+        self._decompressobj: Any = None
+
         if algorithm == "zstd_stream":
             self._compressobj = zstd.ZstdCompressor()
             self._decompressobj = zstd.ZstdDecompressor()
         elif algorithm == "deflate_stream":
             self._compressobj = zlib.compressobj()
             self._decompressobj = zlib.decompressobj()
-        else:
-            self._compressobj = None
-            self._decompressobj = None
 
-    def compress(self, data):
+    def compress(self, data: StrOrBytes) -> bytes:
         """Compresses data and returns it.
 
         Args:
@@ -122,7 +138,7 @@ class Compressor:
         compressed += self._compressobj.flush(zlib.Z_SYNC_FLUSH)
         return compressed
 
-    def decompress(self, data):
+    def decompress(self, data: StrOrBytes) -> bytes:
         """Decompresses a frame of data and returns it as a string of bytes.
 
         Args:
@@ -153,13 +169,13 @@ class MessageReader:
     .. versionadded:: 8.0.21
     """
 
-    def __init__(self, socket_stream):
-        self._stream = socket_stream
-        self._compressor = None
-        self._msg = None
-        self._msg_queue = []
+    def __init__(self, socket_stream: SocketType) -> None:
+        self._stream: SocketType = socket_stream
+        self._compressor: Optional[Compressor] = None
+        self._msg: MessageType = None
+        self._msg_queue: List[Message] = []
 
-    def _read_message(self):
+    def _read_message(self) -> MessageType:
         """Reads X Protocol messages from the stream and returns a
         :class:`mysqlx.protobuf.Message` object.
 
@@ -206,7 +222,7 @@ class MessageReader:
 
         return frame_msg
 
-    def read_message(self):
+    def read_message(self) -> MessageType:
         """Read message.
 
         Returns:
@@ -218,7 +234,7 @@ class MessageReader:
             return msg
         return self._read_message()
 
-    def push_message(self, msg):
+    def push_message(self, msg: MessageType) -> None:
         """Push message.
 
         Args:
@@ -231,7 +247,7 @@ class MessageReader:
             raise OperationalError("Message push slot is full")
         self._msg = msg
 
-    def set_compression(self, algorithm):
+    def set_compression(self, algorithm: str) -> None:
         """Creates a :class:`mysqlx.protocol.Compressor` object based on the
         compression algorithm.
 
@@ -254,11 +270,11 @@ class MessageWriter:
 
     """
 
-    def __init__(self, socket_stream):
-        self._stream = socket_stream
-        self._compressor = None
+    def __init__(self, socket_stream: SocketType) -> None:
+        self._stream: SocketType = socket_stream
+        self._compressor: Optional[Compressor] = None
 
-    def write_message(self, msg_type, msg):
+    def write_message(self, msg_type: int, msg: MessageType) -> None:
         """Write message.
 
         Args:
@@ -295,7 +311,7 @@ class MessageWriter:
             header = struct.pack("<LB", msg_size + 1, msg_type)
             self._stream.sendall(b"".join([header, msg_str]))
 
-    def set_compression(self, algorithm):
+    def set_compression(self, algorithm: str) -> None:
         """Creates a :class:`mysqlx.protocol.Compressor` object based on the
         compression algorithm.
 
@@ -315,19 +331,19 @@ class Protocol:
     .. versionchanged:: 8.0.21
     """
 
-    def __init__(self, reader, writer):
-        self._reader = reader
-        self._writer = writer
-        self._compression_algorithm = None
-        self._warnings = []
+    def __init__(self, reader: MessageReader, writer: MessageWriter) -> None:
+        self._reader: MessageReader = reader
+        self._writer: MessageWriter = writer
+        self._compression_algorithm: Optional[str] = None
+        self._warnings: List[str] = []
 
     @property
-    def compression_algorithm(self):
+    def compression_algorithm(self) -> Optional[str]:
         """str: The compresion algorithm."""
         return self._compression_algorithm
 
     @staticmethod
-    def _apply_filter(msg, stmt):
+    def _apply_filter(msg: MessageType, stmt: FilterableStatement) -> None:
         """Apply filter.
 
         Args:
@@ -343,7 +359,7 @@ class Protocol:
         if stmt.has_having:
             msg["grouping_criteria"] = stmt.get_having()
 
-    def _create_any(self, arg):
+    def _create_any(self, arg: Any) -> Optional[MessageType]:
         """Create any.
 
         Args:
@@ -417,7 +433,9 @@ class Protocol:
 
         return None
 
-    def _get_binding_args(self, stmt, is_scalar=True):
+    def _get_binding_args(
+        self, stmt: Union[FilterableStatement, SqlStatement], is_scalar: bool = True
+    ) -> Union[List[None], List[Union[ProtobufMessageType, ProtobufMessageCextType]]]:
         """Returns the binding any/scalar.
 
         Args:
@@ -432,7 +450,9 @@ class Protocol:
             list: A list of ``Any`` or ``Scalar`` objects.
         """
 
-        def build_value(value):
+        def build_value(
+            value: Any,
+        ) -> Union[ProtobufMessageType, ProtobufMessageCextType]:
             if is_scalar:
                 return build_scalar(value).get_message()
             return self._create_any(value).get_message()
@@ -445,12 +465,12 @@ class Protocol:
             return [build_value(value) for value in bindings]
 
         count = len(binding_map)
-        args = count * [None]
+        args: List[Any] = count * [None]
         if count != len(bindings):
             raise ProgrammingError(
                 "The number of bind parameters and placeholders do not match"
             )
-        for name, value in bindings.items():
+        for name, value in bindings.items():  # type: ignore[union-attr]
             if name not in binding_map:
                 raise ProgrammingError(
                     f"Unable to find placeholder for parameter: {name}"
@@ -459,7 +479,7 @@ class Protocol:
             args[pos] = build_value(value)
         return args
 
-    def _process_frame(self, msg, result):
+    def _process_frame(self, msg: MessageType, result: ResultBaseType) -> None:
         """Process frame.
 
         Args:
@@ -469,7 +489,7 @@ class Protocol:
         if msg["type"] == 1:
             warn_msg = Message.from_message("Mysqlx.Notice.Warning", msg["payload"])
             self._warnings.append(warn_msg.msg)
-            _LOGGER.warning(
+            logger.warning(
                 "Protocol.process_frame Received Warning Notice code %s: %s",
                 warn_msg.code,
                 warn_msg.msg,
@@ -493,13 +513,7 @@ class Protocol:
                     ]
                 )
             else:  # Following results are unitary and not a list
-                sess_state_value = (
-                    sess_state_msg["value"][0]
-                    if isinstance(
-                        sess_state_msg["value"], tuple(PROTOBUF_REPEATED_TYPES)
-                    )
-                    else sess_state_msg["value"]
-                )
+                sess_state_value = sess_state_msg["value"].pop()
                 if sess_state_msg["param"] == mysqlxpb_enum(
                     "Mysqlx.Notice.SessionStateChanged.Parameter.ROWS_AFFECTED"
                 ):
@@ -513,7 +527,7 @@ class Protocol:
                         get_item_or_attr(sess_state_value, "v_unsigned_int")
                     )
 
-    def _read_message(self, result):
+    def _read_message(self, result: ResultBaseType) -> Optional[MessageType]:
         """Read message.
 
         Args:
@@ -546,7 +560,7 @@ class Protocol:
                 break
         return msg
 
-    def set_compression(self, algorithm):
+    def set_compression(self, algorithm: str) -> None:
         """Sets the compression algorithm to be used by the compression
         object, for uplink and downlink.
 
@@ -560,7 +574,7 @@ class Protocol:
         self._reader.set_compression(algorithm)
         self._writer.set_compression(algorithm)
 
-    def get_capabilites(self):
+    def get_capabilites(self) -> MessageType:
         """Get capabilities.
 
         Returns:
@@ -580,7 +594,7 @@ class Protocol:
 
         return msg
 
-    def set_capabilities(self, **kwargs):
+    def set_capabilities(self, **kwargs: Any) -> None:
         """Set capabilities.
 
         Args:
@@ -628,7 +642,12 @@ class Protocol:
                 raise
         return None
 
-    def send_auth_start(self, method, auth_data=None, initial_response=None):
+    def send_auth_start(
+        self,
+        method: str,
+        auth_data: Optional[str] = None,
+        initial_response: Optional[str] = None,
+    ) -> None:
         """Send authenticate start.
 
         Args:
@@ -647,7 +666,7 @@ class Protocol:
             msg,
         )
 
-    def read_auth_continue(self):
+    def read_auth_continue(self) -> bytes:
         """Read authenticate continue.
 
         Raises:
@@ -666,7 +685,7 @@ class Protocol:
             )
         return msg["auth_data"]
 
-    def send_auth_continue(self, auth_data):
+    def send_auth_continue(self, auth_data: str) -> None:
         """Send authenticate continue.
 
         Args:
@@ -678,7 +697,7 @@ class Protocol:
             msg,
         )
 
-    def read_auth_ok(self):
+    def read_auth_ok(self) -> None:
         """Read authenticate OK.
 
         Raises:
@@ -691,7 +710,19 @@ class Protocol:
             if msg.type == "Mysqlx.Error":
                 raise InterfaceError(msg.msg)
 
-    def send_prepare_prepare(self, msg_type, msg, stmt):
+    def send_prepare_prepare(
+        self,
+        msg_type: str,
+        msg: MessageType,
+        stmt: Union[
+            FindStatement,
+            DeleteStatement,
+            ModifyStatement,
+            ReadStatement,
+            RemoveStatement,
+            UpdateStatement,
+        ],
+    ) -> None:
         """
         Send prepare statement.
 
@@ -709,11 +740,11 @@ class Protocol:
         if stmt.has_limit and msg.type != "Mysqlx.Crud.Insert":
             # Remove 'limit' from message by building a new one
             if msg.type == "Mysqlx.Crud.Find":
-                _, msg = self.build_find(stmt)
+                _, msg = self.build_find(stmt)  # type: ignore[arg-type]
             elif msg.type == "Mysqlx.Crud.Update":
-                _, msg = self.build_update(stmt)
+                _, msg = self.build_update(stmt)  # type: ignore[arg-type]
             elif msg.type == "Mysqlx.Crud.Delete":
-                _, msg = self.build_delete(stmt)
+                _, msg = self.build_delete(stmt)  # type: ignore[arg-type]
             else:
                 raise ValueError(f"Invalid message type: {msg_type}")
             # Build 'limit_expr' message
@@ -747,7 +778,9 @@ class Protocol:
         except InterfaceError as err:
             raise NotSupportedError from err
 
-    def send_prepare_execute(self, msg_type, msg, stmt):
+    def send_prepare_execute(
+        self, msg_type: str, msg: MessageType, stmt: FilterableStatement
+    ) -> None:
         """
         Send execute statement.
 
@@ -782,7 +815,7 @@ class Protocol:
             msg_execute,
         )
 
-    def send_prepare_deallocate(self, stmt_id):
+    def send_prepare_deallocate(self, stmt_id: int) -> None:
         """
         Send prepare deallocate statement.
 
@@ -799,7 +832,12 @@ class Protocol:
         )
         self.read_ok()
 
-    def send_msg_without_ps(self, msg_type, msg, stmt):
+    def send_msg_without_ps(
+        self,
+        msg_type: str,
+        msg: MessageType,
+        stmt: Union[FilterableStatement, SqlStatement],
+    ) -> None:
         """
         Send a message without prepared statements support.
 
@@ -812,9 +850,9 @@ class Protocol:
         """
         if stmt.has_limit:
             msg_limit = Message("Mysqlx.Crud.Limit")
-            msg_limit["row_count"] = stmt.get_limit_row_count()
+            msg_limit["row_count"] = stmt.get_limit_row_count()  # type: ignore[union-attr]
             if msg.type == "Mysqlx.Crud.Find":
-                msg_limit["offset"] = stmt.get_limit_offset()
+                msg_limit["offset"] = stmt.get_limit_offset()  # type: ignore[union-attr]
             msg["limit"] = msg_limit
         is_scalar = msg_type != "Mysqlx.ClientMessages.Type.SQL_STMT_EXECUTE"
         args = self._get_binding_args(stmt, is_scalar=is_scalar)
@@ -822,7 +860,7 @@ class Protocol:
             msg["args"].extend(args)
         self.send_msg(msg_type, msg)
 
-    def send_msg(self, msg_type, msg):
+    def send_msg(self, msg_type: str, msg: MessageType) -> None:
         """
         Send a message.
 
@@ -834,7 +872,9 @@ class Protocol:
         """
         self._writer.write_message(mysqlxpb_enum(msg_type), msg)
 
-    def build_find(self, stmt):
+    def build_find(
+        self, stmt: Union[FindStatement, ReadStatement]
+    ) -> Tuple[str, MessageType]:
         """Build find/read message.
 
         Args:
@@ -874,7 +914,9 @@ class Protocol:
 
         return "Mysqlx.ClientMessages.Type.CRUD_FIND", msg
 
-    def build_update(self, stmt):
+    def build_update(
+        self, stmt: Union[ModifyStatement, UpdateStatement]
+    ) -> Tuple[str, MessageType]:
         """Build update message.
 
         Args:
@@ -913,7 +955,9 @@ class Protocol:
 
         return "Mysqlx.ClientMessages.Type.CRUD_UPDATE", msg
 
-    def build_delete(self, stmt):
+    def build_delete(
+        self, stmt: Union[DeleteStatement, RemoveStatement]
+    ) -> Tuple[str, MessageType]:
         """Build delete message.
 
         Args:
@@ -944,7 +988,12 @@ class Protocol:
         self._apply_filter(msg, stmt)
         return "Mysqlx.ClientMessages.Type.CRUD_DELETE", msg
 
-    def build_execute_statement(self, namespace, stmt, fields=None):
+    def build_execute_statement(
+        self,
+        namespace: str,
+        stmt: Union[str, StatementType],
+        fields: Optional[Dict[str, Any]] = None,
+    ) -> Tuple[str, MessageType]:
         """Build execute statement.
 
         Args:
@@ -982,7 +1031,9 @@ class Protocol:
         return "Mysqlx.ClientMessages.Type.SQL_STMT_EXECUTE", msg
 
     @staticmethod
-    def build_insert(stmt):
+    def build_insert(
+        stmt: Union[AddStatement, InsertStatement]
+    ) -> Tuple[str, MessageType]:
         """Build insert statement.
 
         Args:
@@ -1032,7 +1083,7 @@ class Protocol:
 
         return "Mysqlx.ClientMessages.Type.CRUD_INSERT", msg
 
-    def close_result(self, result):
+    def close_result(self, result: ResultBaseType) -> None:
         """Close the result.
 
         Args:
@@ -1045,7 +1096,7 @@ class Protocol:
         if msg is not None:
             raise OperationalError("Expected to close the result")
 
-    def read_row(self, result):
+    def read_row(self, result: ResultBaseType) -> Optional[MessageType]:
         """Read row.
 
         Args:
@@ -1059,7 +1110,7 @@ class Protocol:
         self._reader.push_message(msg)
         return None
 
-    def get_column_metadata(self, result):
+    def get_column_metadata(self, result: ResultBaseType) -> List[ColumnType]:
         """Returns column metadata.
 
         Args:
@@ -1095,7 +1146,7 @@ class Protocol:
             columns.append(col)
         return columns
 
-    def read_ok(self):
+    def read_ok(self) -> None:
         """Read OK.
 
         Raises:
@@ -1107,21 +1158,21 @@ class Protocol:
         if msg.type != "Mysqlx.Ok":
             raise InterfaceError("Unexpected message encountered")
 
-    def send_connection_close(self):
+    def send_connection_close(self) -> None:
         """Send connection close."""
         msg = Message("Mysqlx.Connection.Close")
         self._writer.write_message(
             mysqlxpb_enum("Mysqlx.ClientMessages.Type.CON_CLOSE"), msg
         )
 
-    def send_close(self):
+    def send_close(self) -> None:
         """Send close."""
         msg = Message("Mysqlx.Session.Close")
         self._writer.write_message(
             mysqlxpb_enum("Mysqlx.ClientMessages.Type.SESS_CLOSE"), msg
         )
 
-    def send_expect_open(self):
+    def send_expect_open(self) -> None:
         """Send expectation."""
         cond_key = mysqlxpb_enum("Mysqlx.Expect.Open.Condition.Key.EXPECT_FIELD_EXIST")
         msg_oc = Message("Mysqlx.Expect.Open.Condition")
@@ -1135,7 +1186,7 @@ class Protocol:
             mysqlxpb_enum("Mysqlx.ClientMessages.Type.EXPECT_OPEN"), msg_eo
         )
 
-    def send_reset(self, keep_open=None):
+    def send_reset(self, keep_open: Optional[bool] = None) -> bool:
         """Send reset session message.
 
         Returns:
